/*
 * Copyright (C) 2024 Black Duck Software, Inc.
 * https://www.blackduck.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Black Duck ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Black Duck.
 */
package com.blackduck.artifactory.integration

import com.blackduck.artifactory.integration.configuration.DirectoryConfig
import com.blackduck.artifactory.integration.modules.PluginAPI
import com.blackduck.artifactory.integration.modules.analytics.AnalyticsModule
import groovy.transform.Field
import org.artifactory.fs.ItemInfo
import org.artifactory.repo.RepoPath
import org.artifactory.request.Request

// propertiesFilePathOverride allows you to specify an absolute path to the blackDuckArtifactoryIntegration.properties file.
// If this is empty, we will default to ${ARTIFACTORY_HOME}/etc/plugins/lib/blackDuckArtifactoryIntegration.properties
@Field String propertiesFilePathOverride = ""
@Field PluginService pluginService
@Field PluginAPI pluginAPI

initialize(TriggerType.STARTUP)

executions {
    //////////////////////////////////////////////// PLUGIN EXECUTIONS ////////////////////////////////////////////////

    /**
     * This will attempt to reload the properties file and initialize the plugin with the new values.
     * Does not apply to CRON expressions. An Artifactory restart is required for changing cron expressions.
     *
     * This can be triggered with the following curl command:
     * curl -X POST -u admin:password "http://ARTIFACTORY_SERVER/artifactory/api/plugins/execute/blackDuckArtifactoryIntegrationReload"
     **/
    blackDuckArtifactoryIntegrationReload(httpMethod: 'POST') { params -> initialize(TriggerType.REST_REQUEST)
    }

    /**
     * This will enabled or disable a particular module within the plugin and then reinitialize the plugin.
     * This endpoint requires parameters to be set. Each module has an enabled state of 'true' or 'false'
     * The names for the propertyReports available are:
     *      ScanModule
     *      InspectionModule
     *      AnalyticsModule
     *
     * NOTE: Enabling or disabling an endpoint via this endpoint is only persistent as long as the plugin is loaded.
     *       Set the state in the blackDuckArtifactoryIntegration.properties file to allow the state to survive reinitialization.
     *       All permanent changes made to configuration must be made in the blackDuckArtifactoryIntegration.properties file
     *
     * This can be triggered with the following curl command for setting the state of a module
     * curl -X POST -u admin:password "http://ARTIFACTORY_SERVER/artifactory/api/plugins/execute/blackDuckSetModuleState?params=<moduleName>=<enabledState>"
     *
     * This can be triggered with the following curl command for enabling the InspectionModule:
     * curl -X POST -u admin:password "http://ARTIFACTORY_SERVER/artifactory/api/plugins/execute/blackDuckSetModuleState?params=InspectionModule=true"
     *
     * This can be triggered with the following curl command for disabling multiple propertyReports (ScanModule and the InspectionModule):
     * Note: "%7C" is the url encoding for the pipe '|' character.
     * curl -X POST -u admin:password "http://ARTIFACTORY_SERVER/artifactory/api/plugins/execute/blackDuckSetModuleState?params=ScanModule=false%InspectionModule=false"
     **/
    blackDuckSetModuleState() { params -> pluginAPI.setModuleState(TriggerType.REST_REQUEST, (Map<String, List<String>>) params)
    }

    /**
     * This will return a current status of the scan module's configuration configuration to verify things are setup properly.
     *
     * This can be triggered with the following curl command:
     * curl -X GET -u admin:password "http://ARTIFACTORY_SERVER/artifactory/api/plugins/execute/blackDuckTestConfig"
     **/
    blackDuckTestConfig(httpMethod: 'GET') { params -> message = pluginService.logStatusCheckMessage(TriggerType.REST_REQUEST)
    }

    //////////////////////////////////////////////// ANALYTICS EXECUTIONS ////////////////////////////////////////////////

    /**
     * Submits usage analytics. This endpoint is intended for developer use only
     *
     * This can be triggered with the following curl command:
     * curl -X POST -u admin:password "http://ARTIFACTORY_SERVER/artifactory/api/plugins/execute/blackDuckSubmitAnalytics"
     */
    blackDuckSubmitAnalytics(httpMethod: 'POST') { params -> message = pluginAPI.submitAnalytics(TriggerType.REST_REQUEST)
    }

    //////////////////////////////////////////////// SCAN-AS-A-SERVICE EXECUTIONS ////////////////////////////////////////////////

    /**
     * Removes all properties that were populated by the SCA-as-a-Service function for the repositories listed in the "repos" parameter as long as
     * those repos also exist in the blackduck.artifactory.scaaas.blocking.repos property.
     * If the property blackduck.scaaas.resultsUrl is set on an artifact, the file it points to will be deleted
     *
     * This can be triggered with the following curl command:
     * curl -X POST -u admin:password "http://ARTIFACTORY_SERVER/artifactory/api/plugins/execute/blackDuckDeleteScanAsAServicePropertiesOnRepos?params=repos=repo1[,repo2 ...]"
     */
    blackDuckDeleteScanAsAServicePropertiesOnRepos(httpMethod: 'POST') { params -> message = pluginAPI.performDeleteScanAsAServicePropertiesOnRepos(TriggerType.REST_REQUEST, (Map<String, List<String>>) params)
    }
}

jobs {
    //////////////////////////////////////////////// ARTIFACTORY INTEGRATION JOBS //////////////////////////////////

    /**
     * Search configured repositories for files that need to be scanned. Send files to BlackDuck to be scanned.
     */
    blackDuckArtifactoryIntegrationFindAndScan(interval: pluginAPI.getItemSearchInterval(), delay: 30_000L) {
        pluginAPI.findAndScan(TriggerType.CRON_JOB)
    }

    /**
     * Search for scan results. Update corresponding annotations.
     * Search occurs alternating with findAndScan.
     */
    blackDuckArtifactoryIntegrationGetResults(interval: pluginAPI.getItemSearchInterval(), delay: pluginAPI.getSearchResultDelay()) {
        pluginAPI.findScanResults(TriggerType.CRON_JOB)
    }

    /**
     *
     */
    blackDuckArtifactoryIntegrationCheckForUpdatedConfig(cron: "0 0 */3 ? * *") {
        updateConfig(TriggerType.CRON_JOB);
    }

    /**
     * Connect to Black Duck and wait for component updates.
     */
    blackDuckHandleComponentUpdates(interval: pluginAPI.getComponentUpdateInterval(), delay: 60_000L) {
        pluginAPI.findComponentUpdates(TriggerType.CRON_JOB)
    }

    /**
     * Process results from uploads/scans that were completed after blackDuckArtifactoryIntegrationFindAndScan processing
     * completed.
     */
    blackDuckArtifactoryIntegrationProcessInProgressScansResults(interval: pluginAPI.getProcessInProgressScansResultsInterval()) {
        pluginAPI.processInProgressScansResults(TriggerType.CRON_JOB)
    }

    //////////////////////////////////////////////// ANALYTICS JOBS ////////////////////////////////////////////////

    /**
     * Submits usage analytics. For developer use only
     **/
    blackDuckSubmitAnalytics(cron: AnalyticsModule.SUBMIT_ANALYTICS_CRON) {
        pluginAPI.submitAnalytics(TriggerType.CRON_JOB)
    }
}

storage {
    //////////////////////////////////////////////// INSPECTION STORAGE ////////////////////////////////////////////////

    /**
     * Handle after create events.
     *
     * Closure parameters:
     * item (org.artifactory.fs.ItemInfo) - the original item being created.*/
    afterCreate { ItemInfo item -> pluginAPI.handleAfterCreateEvent(item, TriggerType.STORAGE_AFTER_CREATE)
    }

    /**
     * Handle after copy events.
     *
     * Closure parameters:
     * item (org.artifactory.fs.ItemInfo) - the source item copied.
     * targetRepoPath (org.artifactory.repo.RepoPath) - the target repoPath for the copy.*/
    afterCopy { ItemInfo item, RepoPath targetRepoPath, properties -> pluginAPI.handleAfterCopyEvent(targetRepoPath, TriggerType.STORAGE_AFTER_COPY)
    }

    /**
     * Handle after move events.
     *
     * Closure parameters:
     * item (org.artifactory.fs.ItemInfo) - the source item moved.
     * targetRepoPath (org.artifactory.repo.RepoPath) - the target repoPath for the move.*/
    afterMove { ItemInfo item, RepoPath targetRepoPath, properties -> pluginAPI.handleAfterMoveEvent(targetRepoPath, TriggerType.STORAGE_AFTER_MOVE)
    }
}

download {
    //////////////////////////////////////////////// POLICY ENFORCER ////////////////////////////////////////////////

    beforeDownload { Request request, RepoPath repoPath ->
        pluginAPI.handleBeforeDownloadEventScanAsAService(TriggerType.BEFORE_DOWNLOAD, request, repoPath)
    }
}

private void initialize(final TriggerType triggerType) {
    log.info("Initializing blackDuckArtifactoryIntegration from ${triggerType.getLogName()}...")

    final File etcDirectory = ctx.artifactoryHome.etcDir
    final File homeDirectory = ctx.artifactoryHome.homeDir
    final File pluginsDirectory = ctx.artifactoryHome.pluginsDir
    // Used to be 'versionProvider.running', but that is no longer available, 'originalHomeVersion' is the closest substitute
    final String thirdPartyVersion = ctx?.versionProvider?.originalHomeVersion?.versionName?.toString()
    final DirectoryConfig pluginConfig = DirectoryConfig.createDefault(homeDirectory, etcDirectory, pluginsDirectory, thirdPartyVersion, propertiesFilePathOverride)

    pluginService = new PluginService(pluginConfig, repositories, searches)
    pluginAPI = pluginService.initializePlugin()

    log.info("... completed intialization of blackDuckArtifactoryIntegration from ${triggerType.getLogName()}")
}

private void updateConfig(final TriggerType triggerType) {
    log.info("Checking for updated configuration of blackDuckArtifactoryIntegration from ${triggerType.getLogName()}...")

    pluginService.applyUpdatedConfiguration().ifPresent { updatedAPI -> pluginAPI = updatedAPI }

    log.info("... completed checking for updated configuration of blackDuckArtifactoryIntegration from ${triggerType.getLogName()}")
}

